@@ -133,7 +133,6 @@ extern crate time;
 #[macro_use] extern crate url;
 extern crate unicase;
 extern crate httparse;
-extern crate num_cpus;
 extern crate traitobject;
 extern crate typeable;
 
@@ -164,9 +163,9 @@ macro_rules! todo(
     })
 );
 
-#[cfg(test)]
+//#[cfg(test)]
 #[macro_use]
-mod mock;
+pub mod mock;
 #[doc(hidden)]
 pub mod buffer;
 pub mod client;
