@@ -112,14 +112,9 @@ use std::io::{self, ErrorKind, BufWriter, Write};
 use std::net::{SocketAddr, ToSocketAddrs, Shutdown};
 use std::sync::Arc;
 use std::time::Duration;
-
-use num_cpus;
-
 pub use self::request::Request;
 pub use self::response::Response;
-
 pub use crate::net::{Fresh, Streaming};
-
 use crate::{Error, runtime};
 use crate::buffer::BufReader;
 use crate::header::{Headers, Expect, Connection};
@@ -148,7 +143,7 @@ pub struct Server<L = HttpListener> {
 }
 
 #[derive(Clone, Copy, Debug)]
-struct Timeouts {
+pub struct Timeouts {
     read: Option<Duration>,
     keep_alive: Option<Duration>,
 }
@@ -280,22 +275,21 @@ fn handle_task<H, L>(mut server: Server<L>, handler: H, tasks: usize) -> crate::
     })
 }
 
-struct Worker<H: Handler + 'static> {
+pub struct Worker<H: Handler + 'static> {
     handler: H,
     timeouts: Timeouts,
 }
 
 impl<H: Handler + 'static> Worker<H> {
-    fn new(handler: H, timeouts: Timeouts) -> Worker<H> {
+    pub fn new(handler: H, timeouts: Timeouts) -> Worker<H> {
         Worker {
             handler: handler,
             timeouts: timeouts,
         }
     }
 
-    fn handle_connection<S>(&self, stream: &mut S) where S: NetworkStream + Clone {
+    pub fn handle_connection<S>(&self, stream: &mut S) where S: NetworkStream + Clone {
         debug!("Incoming stream");
-
         self.handler.on_connection_start();
 
         let addr = match stream.peer_addr() {
@@ -305,26 +299,21 @@ impl<H: Handler + 'static> Worker<H> {
                 return;
             }
         };
-
-        let stream2: &mut dyn NetworkStream = &mut stream.clone();
+        //safety will forget copy s
+        let mut s: S = unsafe { std::mem::transmute_copy(stream) };
+        let stream2: &mut dyn NetworkStream = &mut s;
         let mut rdr = BufReader::new(stream2);
         let mut wrt = BufWriter::new(stream);
 
         while self.keep_alive_loop(&mut rdr, &mut wrt, addr) {
-            //TODO set read timeout
-            // if let Err(e) = self.set_read_timeout(*rdr.get_ref(), self.timeouts.keep_alive) {
-            //     info!("set_read_timeout keep_alive {:?}", e);
-            //     break;
-            // }
+            if let Err(e) = self.set_read_timeout(*rdr.get_ref(), self.timeouts.keep_alive) {
+                info!("set_read_timeout keep_alive {:?}", e);
+                break;
+            }
         }
-
         self.handler.on_connection_end();
-
         debug!("keep_alive loop ending for {}", addr);
-
-        if let Err(e) = rdr.get_mut().close(Shutdown::Both) {
-            info!("failed to close stream: {}", e);
-        }
+        std::mem::forget(s);
     }
 
     fn set_read_timeout(&self, s: &dyn NetworkStream, timeout: Option<Duration>) -> io::Result<()> {
